/*
 *  USBSink - filetransfer.cc
 *
 *  Copyright (C) 2007 Marko Anastasov
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "filetransfer.h"
#include "logger.h"
#include "task.h"
#include "utils.h"

#include <list>

#include <glibmm/refptr.h>

#include <libgnomevfs/gnome-vfs-result.h>
#include <libgnomevfsmm/uri.h>

namespace USBSink
{

namespace Transfer
{

using std::list;

typedef list< Glib::RefPtr<const Gnome::Vfs::Uri> > uri_list;

typedef Glib::ListHandle< Glib::RefPtr<const Gnome::Vfs::Uri> > ListHandleUris;


FileTransfer::FileTransfer(const Glib::ustring& src_path,
                           const Glib::ustring& dest_path)
  :
  src_path_(src_path),
  dest_path_(dest_path)
{
}

FileTransfer::~FileTransfer()
{
}

void
FileTransfer::run(Logger* logger,
                  TaskHandling::Task* task,
                  GnomeVFSAsyncXferProgressCallback callback)
{
  uri_list source_uris, target_uris;

  *logger << Util::Time::get_current_time() << ": "
          << "Preparing to copy file:\n"
          << src_path_ << "\n"
          << "to path " << dest_path_ << "\n";

  source_uris.push_back(Gnome::Vfs::Uri::create(src_path_));
  target_uris.push_back(Gnome::Vfs::Uri::create(dest_path_));

  ListHandleUris source_proxy(source_uris);
  ListHandleUris target_proxy(target_uris);
  GnomeVFSResult xfer_result;
  CallbackData* callback_data = new CallbackData;
    
  callback_data->logger = logger;
  callback_data->task = task;

  task->inc_transfer_count();

  xfer_result = gnome_vfs_async_xfer(&async_handle_,
                                     source_proxy.data(),
                                     target_proxy.data(),
                                     GNOME_VFS_XFER_DEFAULT,
                                     GNOME_VFS_XFER_ERROR_MODE_QUERY,
                                     GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE,
                                     GNOME_VFS_PRIORITY_DEFAULT,
                                     callback,
                                     callback_data,
                                     NULL,   /* progress_sync_callback */
                                     NULL);  /* sync_callback_data */

  if (xfer_result != GNOME_VFS_OK)
  {
    *logger << "Transfer failed: "
            << gnome_vfs_result_to_string(xfer_result) << "\n";

    g_warning("gnome-vfs transfer failed: %s",
              gnome_vfs_result_to_string(xfer_result));

    task->dec_transfer_count();
  }

  // TODO: wait for libgnomevfsmm release with async2 and transfer2.
}

} // namespace Transfer

} // namespace USBSink
