/*
 *  USBSink - filewatcher.h
 *
 *  Copyright (C) 2006 Marko Anastasov
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _FILE_WATCHER_H_
#define _FILE_WATCHER_H_

#include "uncopyable.h"

#include <fam.h>

#include <memory>
#include <vector>
#include <map>

#include <tr1/memory>

#include <glibmm/main.h>
#include <glibmm/ustring.h>

#include <sigc++/sigc++.h>

/**
 * Watch (monitor) files and directories for changes using FAM.
 * Monitoring can be stopped and restarted again.
 *
 * This is a singleton.
 *
 */

namespace USBSink
{

enum FileStatus
{
  FILE_STATUS_NO_CHANGE = 0,
  FILE_STATUS_CHANGED = 1,
  FILE_STATUS_DELETED = 2,
  FILE_STATUS_CREATED = 4
};

class FileWatcher : private Uncopyable
{
 public:
  
  static FileWatcher* instance();
  ~FileWatcher();

  void add_file(const Glib::ustring& path);
  void remove_file(const Glib::ustring& path);
  
  /**
   * This is used as:
   * void on_monitor(const Glib::ustring& monitor_path,
   *                 const Glib::ustring& info_path,
   *                 FileStatus news);
   */
  sigc::signal<void, Glib::ustring, Glib::ustring, FileStatus>
    signal_monitor;
  
 protected:

  /**
   * FileWatcher's private representation of monitored files and directories.
   * FAMRequests are stored here.
   */
  class Entry
  {
  public:
    explicit Entry(const Glib::ustring& path)
      : path_(path),
      dirty_(false),
      status_(FILE_STATUS_NO_CHANGE),
      is_dir_(false) {}

    ~Entry() {}
    
    FAMRequest& get_request() { return fr_; }
    Glib::ustring get_path() const { return path_; }
    bool is_dirty() const { return dirty_; }
    FileStatus get_change_news() const { return status_; }
    bool is_dir() const { return is_dir_; }    
      
    void set_is_dir(bool val) { is_dir_ = val; }
    void set_change_news(FileStatus news) { status_ = news; }
    void set_dirty(bool to_val) { dirty_ = to_val; }
      
  private:
    Glib::ustring path_;
    FAMRequest fr_;
    bool dirty_;
    FileStatus status_;
    bool is_dir_;
  };
  
  FileWatcher();
  
  bool open_fam();
  void close_fam();
  
  void setup_fam_for_entry(std::tr1::shared_ptr<Entry>& e);
  void remove_all_entries() { files_to_monitor_.clear(); }

  /* bool stop_watching_entry(std::tr1::shared_ptr<Entry> e); */
  /* bool restart_watching_entry(std::tr1::shared_ptr<Entry> e); */

  std::tr1::shared_ptr<Entry> get_entry(const Glib::ustring& path);

  bool on_fam_event(Glib::IOCondition condition);

 private:

  static std::auto_ptr<FileWatcher> instance_;
  
  FAMConnection fam_;
  FAMRequest main_fam_request_;
  
  sigc::connection fam_signal_connection_;
  
  bool started_;
  
  typedef std::map< Glib::ustring, std::tr1::shared_ptr<Entry> > EntryMap;

  EntryMap files_to_monitor_;
};

class FamError // exception type
{
 public:
  FamError(const Glib::ustring& error_msg) : msg_(error_msg) {}
  Glib::ustring what() const { return msg_; }
 private:
  Glib::ustring msg_;
};

} // namespace USBSink

#endif // _FILE_WATCHER_H_
