/*
 *  USBSink - flashdrive.cc
 *
 *  Copyright (C) 2005 Marko Anastasov
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "flashdrive.h"
#include "halclient.h"

#include <sys/statvfs.h>

#include <string>

#include <glib/gmessages.h>

#include <dbus/dbus-glib-lowlevel.h>

#include <libhal.h>
#include <libhal-storage.h>

using std::string;

namespace USBSink
{

FlashDrive::FlashDrive(DBusConnection* dbus_connection,
		       LibHalDrive* drive,
		       LibHalVolume* volume)
  :
  dbus_conn_(dbus_connection),
  is_mounted_(false)
{
  udi_ = libhal_volume_get_udi(volume);

//  g_debug("FlashDrive::FlashDrive(): creating a new object for %s",
//          udi_.c_str());

  load_dev_info(drive, volume);
}

FlashDrive::~FlashDrive()
{}

void
FlashDrive::load_dev_info(LibHalDrive* drive, LibHalVolume* volume)
{
  const char* str;

//  g_debug("FlashDrive::load_dev_info()");
  
  // Can't pass possible null directly to string's ctor.
  if ((str = libhal_volume_get_label(volume)))
  {
    label_ = str;
  }
  else
  {
    // There are volumes without a label,
    // so we're gonna set it to drive model, which should make sense.
    label_ = libhal_drive_get_model(drive);
  }

//  g_debug("label is %s", label_.c_str());
  
  if ((is_mounted_ = libhal_volume_is_mounted(volume)))
  {
    // It's safe to get the following info as well.
    fstype_ = libhal_volume_get_fstype(volume);
    mount_point_ = libhal_volume_get_mount_point(volume);

//    g_debug("volume mounted at %s; fstype: %s",
//            mount_point_.c_str(), fstype_.c_str());
  }
  else
  {
    g_debug("volume not mounted");
  }
  
  size_ = static_cast<unsigned>(libhal_volume_get_size(volume));
//  g_debug("volume size is %u", size_);

  vendor_ = libhal_drive_get_vendor(drive);
  model_ = libhal_drive_get_model(drive);
}

/** 
 * Mostly taken from gnome-volume-manager.
 */
bool
FlashDrive::mount()
{
  char* fstype;
  char* mount_point;
  char* options[0];
  int noptions = 0;

  DBusMessage *msg, *reply;
  DBusError error;
	
  g_debug("mounting %s...", udi_.c_str());
  
  if (! (msg = dbus_message_new_method_call(
	   "org.freedesktop.Hal",
	   udi_.c_str(),
	   "org.freedesktop.Hal.Device.Volume",
	   "Mount")))
  {
    g_debug("mount failed for %s: could not create dbus message.",
	    udi_.c_str());
    throw MountError("Error while creating dbus message.");
    return false;
  }
  
  fstype = "";
  mount_point = "";
  options[0] = NULL;
  noptions = 0;
  
  if (! dbus_message_append_args(msg,
				 DBUS_TYPE_STRING, &mount_point,
				 DBUS_TYPE_STRING, &fstype,
				 DBUS_TYPE_ARRAY, DBUS_TYPE_STRING,
				 &options, noptions,
				 DBUS_TYPE_INVALID))
  {
    g_debug("mount failed for %s: could not append args to dbus message",
	    udi_.c_str());
    dbus_message_unref(msg);
    throw MountError("Error while creating dbus message.");
    return false;
  }

  dbus_error_init (&error);
  
  if (! (reply = dbus_connection_send_with_reply_and_block(dbus_conn_,
							   msg,
							   -1,
							   &error)))
  {
    g_debug("mount failed for %s: %s", udi_.c_str(), error.message);
    string details(error.message);
    throw MountError(details);
    dbus_message_unref(msg);
    dbus_error_free(&error);
    return false;
  }
  
  g_debug("mount command succeeded for %s", udi_.c_str());
  
  dbus_message_unref(msg);
  dbus_message_unref(reply);
  
  return true;
}

bool
FlashDrive::umount()
{
  DBusMessage *msg, *reply;
  DBusError error;
  char **options = NULL;
  unsigned int num_options = 0;
  
  g_debug("unmounting %s...", udi_.c_str());
  
  if (! (msg = dbus_message_new_method_call(
	   "org.freedesktop.Hal",
	   udi_.c_str(),
	   "org.freedesktop.Hal.Device.Volume",
	   "Unmount")))
  {
    g_warning("unmount failed for %s: could not create dbus message",
	      udi_.c_str());
    throw MountError("Error while creating dbus message.");
    return false;
  }
  
  if (! dbus_message_append_args(msg, 
				 DBUS_TYPE_ARRAY, DBUS_TYPE_STRING,
				 &options, num_options,
				 DBUS_TYPE_INVALID))
  {
    g_warning("Could not append args to dbus message for %s", udi_.c_str());
    dbus_message_unref(msg);
    throw MountError("Error while creating dbus message.");
    return false;
  }
	
  dbus_error_init (&error);
  
  if (! (reply = dbus_connection_send_with_reply_and_block(dbus_conn_,
							   msg,
							   -1,
							   &error)))
  {
    g_warning("unmount failed for %s: %s", udi_.c_str(), error.message);
    string details(error.message);
    throw MountError(details);
    dbus_message_unref(msg);
    dbus_error_free(&error);
    return false;
  }
  
  g_debug("unmount command succeeded for %s", udi_.c_str());
  
  dbus_message_unref(msg);
  dbus_message_unref(reply);
  
  return true;
}
  
// Returns free space in bytes.
unsigned
FlashDrive::get_free_space()
{
  struct statvfs fsi;
  
  if (statvfs(mount_point_.c_str(), &fsi) != 0)
  {
    return 0;
  }
  
  return (fsi.f_bfree * fsi.f_bsize);
}
  
// Modified tedious work from gnome-mount.
Glib::ustring
FlashDrive::get_mount_error_details(const char* error_name,
				    const char* /* error_message */)
{
  /* One of the following errors may occur:
   *
   *   org.freedesktop.Hal.Device.Volume.PermissionDenied
   *   org.freedesktop.Hal.Device.Volume.AlreadyMounted
   *   org.freedesktop.Hal.Device.Volume.InvalidMountOption
   *   org.freedesktop.Hal.Device.Volume.FailedToCreateMountpoint
   *   org.freedesktop.Hal.Device.Volume.UnknownFilesystemType
   *   org.freedesktop.Hal.Device.Volume.UnknownFailure
   */

  Glib::ustring msg;
  
  if (strcmp(
	error_name,
	"org.freedesktop.Hal.Device.Volume.PermissionDenied") == 0)
  {
    msg = "You are not privileged to mount the volume ";
    msg.append(label_);
    msg.append(".");
  }
  else if (strcmp(
	     error_name,
	     "org.freedesktop.Hal.Device.Volume.AlreadyMounted") == 0)
  {
    msg = "The volume ";
    msg.append(label_);
    msg.append(" is already mounted.");
  }
  else if (strcmp(
	     error_name,
	     "org.freedesktop.Hal.Device.Volume.InvalidMountOption") == 0)
  {
    msg = "Invalid mount option when attempting to mount the volume ";
    msg.append(label_);
  }
  else if (strcmp(
	     error_name,
	     "org.freedesktop.Hal.Device.Volume.UnknownFilesystemType") == 0)
  {
    msg = "The volume ";
    msg.append(label_);
    msg.append(" uses the ");
    msg.append(fstype_);
    msg.append(" file system which is not supported by your system.");
  }
  else if (strcmp(
	     error_name,
	     "org.freedesktop.Hal.Device.Volume.FailedToCreateMountPoint") == 0
	   || strcmp(
	     error_name,
	     "org.freedesktop.Hal.Device.Volume.UnknownFailure") == 0)
  {
    msg = "Unable to mount the volume ";
    msg.append(label_);
    msg.append(".");
  }
  
  return msg;
}

Glib::ustring
FlashDrive::get_unmount_error_details(const char* error_name,
				      const char* error_message)
{
  /* One of  the following errors maye occur:
   *
   *   org.freedesktop.Hal.Device.Volume.PermissionDenied
   *   org.freedesktop.Hal.Device.Volume.UnsupportedUnmountOption
   *   org.freedesktop.Hal.Device.Volume.InvalidUnmountOption
   *   org.freedesktop.Hal.Device.Volume.Busy
   *   org.freedesktop.Hal.Device.Volume.NotMounted
   *   org.freedesktop.Hal.Device.Volume.UnknownFailure
   */
  
  Glib::ustring msg;
  g_warning("%s\n%s", error_name, error_message);
  
  if (strcmp(
	error_name,
	"org.freedesktop.Hal.Device.Volume.PermissionDenied") == 0)
  {
    msg = "You are not privileged to unmount the volume ";
    msg.append(label_);
    msg.append(".");
  }
  else if (strcmp(error_name, "org.freedesktop.Hal.Device.Volume.Busy") == 0)
  {
    msg = "An application is preventing the volume ";
    msg.append(label_);
    msg.append(" from being unmounted.");
  }
  else if (strcmp(error_name,
		  "org.freedesktop.Hal.Device.Volume.NotMounted") == 0 )
  {
    msg = "The volume ";
    msg.append(label_);
    msg.append(" is not mounted.");
  }
  else if (strcmp(
	     error_name,
	     "org.freedesktop.Hal.Device.Volume.InvalidUnmountOption") == 0
	   || strcmp(
	     error_name,
	     "org.freedesktop.Hal.Device.Volume.UnsupportedUnmountOption") == 0
	   || strcmp(
	     error_name,
	     "org.freedesktop.Hal.Device.Volume.UnknownFailure") == 0)
  {
    msg = "Unable to unmount the volume ";
    msg.append(label_);
    msg.append(".");
  }

  return msg;
}

} // namespace USBSink
