// -*- c++ -*-
/*
 *  USBSink - mainwindow.h
 *
 *  Copyright (C) 2006-2007 Marko Anastasov
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _MAINWINDOW_H_
#define _MAINWINDOW_H_

#include <memory>

#include <tr1/memory>

#include <gtkmm.h>
#include <sigc++/sigc++.h>
#include <libglademm/xml.h>

namespace USBSink
{

typedef std::tr1::shared_ptr<FlashDrive> SharedFlashDrive;

class EditTaskDialog;
class FlashDrivesDialog;
class FileListEditor;
class LogViewDialog;
class NewTaskDialog;
class PrefDialog;
namespace TaskHandling { class Task; }

enum ShowTasks
{
  SHOW_TASKS_ALL,
  SHOW_TASKS_RUNNING,
  SHOW_TASKS_STOPPED
};

class MainWindow : public sigc::trackable
{
public:
  
  MainWindow();
  virtual ~MainWindow();

  Gtk::Window* get_window() { return window_.get(); }
  
protected:

  class ModelColumns : public Gtk::TreeModel::ColumnRecord
  {
  public:
    ModelColumns()
      { add(col_id); add(col_status); add(col_name); add(col_drive_label); }

    Gtk::TreeModelColumn<int> col_id;
    Gtk::TreeModelColumn<Glib::ustring> col_status; // stock icon id
    Gtk::TreeModelColumn<Glib::ustring> col_name;
    Gtk::TreeModelColumn<Glib::ustring> col_drive_label;
  };

  // member functions
  void load_xml();
  void connect_signals();

  void setup_treeview();
  Gtk::TreeModel::Children::iterator get_tree_iter_for_task(int id);
  void display_task(const std::tr1::shared_ptr<TaskHandling::Task>& task);

  void show_tasks(ShowTasks choice);

  // signal handlers

  // ui
  void on_new_task();
  void on_new_task_hide();

  void on_view_file_list_editor();
  void on_file_list_editor(const Glib::ustring& list_name);
  void on_file_list_editor_hide();
  
  void on_edit_task();
  void on_edit_task_hide();
  void on_delete_task();

  void on_select_all();
  
  void on_start();
  void on_stop();

  void on_view_log();
  void on_log_view_dialog_hide();

  void on_show_all_tasks();
  void on_show_running_tasks();
  void on_show_stopped_tasks();
  void on_available_drives();
  void on_available_drives_hide();

  void on_help();  
  void on_about();
  void on_about_dialog_hide();

  void on_quit();

  // application
  void on_task_added(const std::tr1::shared_ptr<TaskHandling::Task>& task);
  void on_task_removed(int task_id);
  void on_task_started(TaskHandling::Task* t);
  void on_task_stopped(TaskHandling::Task* t, const Glib::ustring& note);
  void on_task_begin_sync(TaskHandling::Task* t);
  void on_task_end_sync(TaskHandling::Task* t);

  void on_drive_mounted(const SharedFlashDrive& sfd);
  void on_drive_unmounted(const SharedFlashDrive& sfd);

  std::auto_ptr<Gtk::Window> window_;

  // dialogs
  std::auto_ptr<NewTaskDialog>     new_task_dialog_;
  std::auto_ptr<EditTaskDialog>    edit_task_dialog_;
  std::auto_ptr<Gtk::Dialog>       about_dialog_;
  std::auto_ptr<FlashDrivesDialog> flash_drives_dialog_;
  std::auto_ptr<FileListEditor>    file_list_editor_;
  std::auto_ptr<LogViewDialog>     log_view_dialog_;

  // menu items
  Gtk::ImageMenuItem* menu_item_file_new_task_;
  Gtk::ImageMenuItem* menu_item_file_quit_;
  Gtk::ImageMenuItem* menu_item_edit_task_;
  Gtk::MenuItem*      menu_item_edit_select_all_;
  Gtk::ImageMenuItem* menu_item_edit_delete_task_;
  Gtk::RadioMenuItem* menu_item_view_all_tasks_;
  Gtk::RadioMenuItem* menu_item_view_running_tasks_;
  Gtk::RadioMenuItem* menu_item_view_stopped_tasks_;
  Gtk::MenuItem*      menu_item_view_available_drives_;
  Gtk::MenuItem*      menu_item_view_file_lists_;
  Gtk::MenuItem*      menu_item_view_transfer_log_;
  Gtk::ImageMenuItem* menu_item_actions_start_;
  Gtk::ImageMenuItem* menu_item_actions_stop_;
  Gtk::ImageMenuItem* menu_item_help_contents_;
  Gtk::ImageMenuItem* menu_item_help_about_;

  // toolbar
  Gtk::Toolbar* toolbar_;

  // toolbuttons
  Gtk::ToolButton* new_button_;
  Gtk::ToolButton* edit_button_;
  Gtk::ToolButton* delete_button_;
  Gtk::ToolButton* start_button_;
  Gtk::ToolButton* stop_button_;

  // treeview
  ModelColumns task_columns_;
  Glib::RefPtr<Gtk::ListStore> task_treemodel_;
  Gtk::TreeView* task_view_;
  Glib::RefPtr<Gtk::TreeSelection> task_selection_;

  // status bar
  Gtk::Statusbar* statusbar_;
};

} // namespace USBSink

#endif
