/*
 *  USBSink - taskfileparser.cc
 *
 *  Copyright (C) 2006 Marko Anastasov
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "blackfilelist.h"
#include "filelistregistry.h"
#include "halclient.h"
#include "task.h"
#include "taskfileparser.h"

#include <algorithm>
#include <cstdlib>
#include <sstream>
#include <string>

#include <libgnomevfsmm.h>

using std::list;
using std::ostringstream;

namespace USBSink
{

namespace TaskHandling
{

typedef std::tr1::shared_ptr<Task> SharedTask;

typedef list< Glib::RefPtr<Gnome::Vfs::Uri> > uri_list;

static const Glib::ustring el_root = "usbsink-task";
static const Glib::ustring el_id = "id";
static const Glib::ustring el_name = "name";
static const Glib::ustring el_drive_udi = "drive_udi";
static const Glib::ustring el_data_flow = "data_flow";
static const Glib::ustring el_start_time = "start_time";
static const Glib::ustring el_timeout = "timeout";
static const Glib::ustring el_src_file = "source_file";
static const Glib::ustring el_target_dir = "target_directory";
static const Glib::ustring el_last_state = "last_state";

Glib::ustring
markup_error_code_to_string(Glib::MarkupError::Code code)
{
  Glib::ustring msg;

  // Message text copied and pasted from the glibmm docs.

  switch (code)
  {
  case Glib::MarkupError::BAD_UTF8:
    msg = "Text being parsed was not valid UTF-8";
    break;
  case Glib::MarkupError::EMPTY:
    msg = "Document contained nothing, or only whitespace.";
    break;
  case Glib::MarkupError::PARSE:
    msg = "Document was ill-formed.";
    break;
  case Glib::MarkupError::UNKNOWN_ELEMENT:
    msg = "Element wasn't known.";
    break;
  case Glib::MarkupError::UNKNOWN_ATTRIBUTE:
    msg = "Attribute wasn't known.";
    break;
  case Glib::MarkupError::INVALID_CONTENT:
    msg = "Something was wrong with contents of the document.";
    break;
  }

  return msg;
}

SharedTask
FileParser::TaskArgs::create_task()
{
  // Load the associated file list:

  TaskHandling::FileListRegistry* registry;

  registry = TaskHandling::FileListRegistry::instance();

  try
  {
    file_list =
      std::tr1::static_pointer_cast<FileList>(registry->get_file_list(name));
  }
  catch (FileListNotFoundException& ex)
  {
    g_warning("File list '%s' not found, creating empty one", name.c_str());

    // register a new one assuming black is default

    TaskHandling::file_list_factory factory =
      &TaskHandling::BlackFileList::create;

    file_list = std::tr1::static_pointer_cast<FileList>(
      registry->register_new_file_list(name, factory));
  }

  SharedTask task(
    new TaskHandling::Task(id, name, drive_udi, data_flow, file_list,
                           when_to_start, timeout, src_files, target_dir));

  return task;
}

/**
 * Task file structure:
 *
 * id
 * name
 * drive_udi
 * data_flow
 * start_time
 * timeout
 * source_file paths
 * target_directory path
 * last_state (1 if task was running on previous app exit)
 */

FileParser::FileParser()
{}

FileParser::~FileParser()
{}

void
FileParser::on_start_element(Glib::Markup::ParseContext& /* context */,
                             const Glib::ustring& element_name,
                             const Glib::Markup::Parser::AttributeMap&
                                                         /* attributes */)
{
  if (element_name == el_root)
    task_args_ = new TaskArgs;
}

void
FileParser::on_end_element(Glib::Markup::ParseContext& /* context */,
                           const Glib::ustring& element_name)
{
  if (element_name == el_root)
  {
    SharedTask new_task = task_args_->create_task();
    signal_task_loaded.emit(new_task, task_args_->last_state);
    delete task_args_;
  }
}

void
FileParser::on_text(Glib::Markup::ParseContext& context,
                    const Glib::ustring& text)
{
  if (context.get_element() == el_id)
  {
    task_args_->id = atoi(text.c_str());
  }
  else if (context.get_element() == el_name)
  {
    task_args_->name = text;
  }
  else if (context.get_element() == el_drive_udi)
  {
    task_args_->drive_udi = text.raw();
  }
  else if (context.get_element() == el_data_flow)
  {
    task_args_->data_flow = static_cast<DataFlow>(atoi(text.c_str()));
  }
  else if (context.get_element() == el_start_time)
  {
    task_args_->when_to_start =
      static_cast<StartTimeOption>(atoi(text.c_str()));
  }
  else if (context.get_element() == el_timeout)
  {
    task_args_->timeout = atoi(text.c_str());
  }
  else if (context.get_element() == el_src_file)
  {
    Glib::RefPtr<Gnome::Vfs::Uri> uri = Gnome::Vfs::Uri::create(text);
    task_args_->src_files.push_back(uri);
  }
  else if (context.get_element() == el_target_dir)
  {
    task_args_->target_dir = text;
  }
  else if (context.get_element() == el_last_state)
  {
    (atoi(text.c_str()) == 1)
      ? task_args_->last_state = true
      : task_args_->last_state = false;
  }
}

void
FileParser::on_error(Glib::Markup::ParseContext& /* context */,
                     const Glib::MarkupError& error)
{
  g_warning("Parser error: %s",
            markup_error_code_to_string(error.code()).c_str());
}

void
FileParser::write_start_element(int lvl, const Glib::ustring& name)
{
  for (int i = 0; i < lvl; i++)
  {
    channel_->write("  ");
  }
  channel_->write("<" + name);
}

void
FileParser::write_end_element(int lvl, const Glib::ustring& name)
{
  for (int i = 0; i < lvl; i++)
  {
    channel_->write("  ");
  }
  channel_->write("</" + name + ">\n");
}

void
FileParser::write_attribute(const Glib::ustring& name,
                            const Glib::ustring& value)
{
  channel_->write(" " + name + "=\"");
  channel_->write(Glib::Markup::escape_text(value) + "\"");
}

void
FileParser::write_task_id(const int id)
{
  write_start_element(1, el_id);
  channel_->write(">");

  ostringstream stream;
  stream << id;

  channel_->write(stream.str());
  write_end_element(0, el_id);
}

void
FileParser::write_task_name(const Glib::ustring& value)
{
  write_start_element(1, el_name);
  channel_->write(">");
  channel_->write(value);
  write_end_element(0, el_name);
}

void
FileParser::write_drive_udi(const Glib::ustring& value)
{
  write_start_element(1, el_drive_udi);
  channel_->write(">");
  channel_->write(value);
  write_end_element(0, el_drive_udi);
}

void
FileParser::write_data_flow_type(DataFlow flow)
{
  write_start_element(1, el_data_flow);
  channel_->write(">");

  ostringstream stream;
  stream << flow;

  channel_->write(stream.str());
  write_end_element(0, el_data_flow);
}

void
FileParser::write_start_option(StartTimeOption option)
{
  write_start_element(1, el_start_time);
  channel_->write(">");

  ostringstream stream;
  stream << option;

  channel_->write(stream.str());
  write_end_element(0, el_start_time);
}

void
FileParser::write_timeout(int value)
{
  write_start_element(1, el_timeout);
  channel_->write(">");

  ostringstream stream;
  stream << value;

  channel_->write(stream.str());
  write_end_element(0, el_timeout);
}

void
FileParser::write_source_files(const uri_list& src_files)
{
  uri_list::const_iterator it(src_files.begin());
  uri_list::const_iterator end(src_files.end());
  
  for (; it != end; ++it)
  {
    if (*it) // there's a number of nulls in the list
    {
      write_start_element(1, el_src_file);
      channel_->write(">");
      channel_->write((*it)->get_path());
      write_end_element(0, el_src_file);
    }
  }
}

void
FileParser::write_target_dir(const Glib::ustring& value)
{
  write_start_element(1, el_target_dir);
  channel_->write(">");
  channel_->write(value);
  write_end_element(0, el_target_dir);
}

void
FileParser::write_last_state(bool value)
{
  write_start_element(1, el_last_state);
  channel_->write(">");
  (value) ? channel_->write("1") : channel_->write("0");
  write_end_element(0, el_last_state);
}

/**
 * Client code should catch Glib::IOChannel::create_from_file()'s FileError.
 */
void
FileParser::save_task(const SharedTask& task)
{
  channel_ = Glib::IOChannel::create_from_file(task->get_filename(), "w");

  write_start_element(0, el_root);
  channel_->write(">\n");

  write_task_id(task->get_id());
  write_task_name(task->get_name());
  write_drive_udi(task->get_drive_udi());
  write_data_flow_type(task->get_data_flow());
  write_start_option(task->get_start_time_option());
  write_timeout(task->get_timeout());
  write_source_files(task->get_source_files());
  write_target_dir(task->get_target_dir());
  write_last_state(task->is_running());
  
  write_end_element(0, el_root);
  
  channel_->close();
}

} // namespace TaskHandling

} // namespace USBSink
