/*
 *  USBSink - whitefilelist.cc
 *
 *  Copyright (C) 2006 Marko Anastasov
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "taskitem.h"
#include "utils.h"
#include "whitefilelist.h"

using std::list;

namespace USBSink
{

namespace TaskHandling
{

Glib::ustring WhiteFileList::type_string_ = white_file_list_type_string;

WhiteFileList::WhiteFileList(const Glib::ustring& name)
  :
  FileListBase(name)
{
  save();
}

WhiteFileList::WhiteFileList(const Glib::ustring& name,
                             const Glib::RefPtr<Gtk::TextBuffer>& buffer)
  :
  FileListBase(name,
               buffer,
               sigc::mem_fun(*this,
                             &WhiteFileList::on_registry_request_response))
{
  save();
}

SharedFileList
WhiteFileList::create(const Glib::ustring& name)
{
  return SharedWhiteFileList(new WhiteFileList(name));
}

SharedFileList
WhiteFileList::create(const Glib::ustring& name,
                      const Glib::RefPtr<Gtk::TextBuffer>& text_buffer)
{
  return SharedWhiteFileList(new WhiteFileList(name, text_buffer));
}

SharedFileList
WhiteFileList::create_from_file(const Glib::ustring& path)
{
  SharedWhiteFileList file_list;
  Glib::ustring name(Glib::path_get_basename(path));

  list<Glib::ustring> file_contents(Util::String::load_file_to_list(path));

  // assume that the first two lines are type + empty line
  if (file_contents.size() < 2)
  {
    // don't bother, create it empty
    file_list.reset(new WhiteFileList(name));
    return file_list;
  }

  Glib::RefPtr<Gtk::TextBuffer> text_buffer = Gtk::TextBuffer::create();

  list<Glib::ustring>::iterator it(file_contents.begin());
  list<Glib::ustring>::iterator end(file_contents.end());

  // blindly skip first line, the type indicator.
  ++it;
  
  while (it != end)
  {
    text_buffer->insert(text_buffer->end(), *it + '\n');
    ++it;
  }

  file_list.reset(new WhiteFileList(name, text_buffer));

  return file_list;
}

/**
 * Adds a new Item into the list and saves the list itself immediately.
 *
 * @param new_item An Item to add into the list.
 */
void
WhiteFileList::add(SharedItem& new_item)
{
  FileListBase::add(new_item);
  save();
}

void
WhiteFileList::save() const
{
  //TODO: think of a solution to share this code with BFL

  Glib::RefPtr<Glib::IOChannel> channel;

  try
  {
    channel = Glib::IOChannel::create_from_file(get_path(), "w");
    
    // the "header" consists of the string identifying list type
    // and an empty line
    channel->write(this->get_type_string());
    channel->write("\n\n");
    
    list<SharedItem>::const_iterator it(FileListBase::begin());
    list<SharedItem>::const_iterator end(FileListBase::end());
    
    for ( ; it != end; ++it)
      channel->write((*it)->get_name_formatted() + '\n');
    
    channel->close();
  }
  catch (Glib::FileError& ex)
  {
    g_warning("Could not open task file list '%s': %s",
              get_name().c_str(), ex.what().c_str());
    //TODO: notify in UI?
  }
}

bool
WhiteFileList::filter(const Glib::ustring& str) const
{
  bool result = false;

  list<SharedItem>::const_iterator it(FileListBase::begin());
  list<SharedItem>::const_iterator end(FileListBase::end());
    
  for ( ; it != end; ++it)
  {
    if ((*it)->match(str))
    {
      result = true;
      break;
    }
  }

  return result;
}

void
WhiteFileList::on_registry_request_response(SharedItem& item)
{
  add(item);
  save();
}

/**
 * Assigns new items to the list and saves it.
 *
 * @param new_items New items that should make up the list.
 */
void
WhiteFileList::set_items(const std::list<SharedItem>& new_items)
{
  FileListBase::set_items(new_items);
  save();
}

} // namespace TaskHandling

} // namespace USBSink
